/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.robokind.demo.robot.replication;

import org.robokind.api.messaging.services.ServiceError;
import org.jflux.api.core.event.Event;
import org.jflux.api.core.event.Header;
import org.jflux.api.core.node.ConsumerNode;
import java.util.Arrays;
import java.util.List;
import org.jflux.api.core.Listener;
import org.jflux.api.core.config.Configuration;
import org.jflux.api.core.node.DefaultConsumerNode;
import org.jflux.api.core.util.ListMergeAdapter;
import org.robokind.api.common.lifecycle.config.RKManagedGroupConfigUtils;
import org.robokind.api.common.lifecycle.utils.ManagedServiceFactory;
import org.robokind.api.common.lifecycle.utils.ManagedServiceGroup;
import org.robokind.api.messaging.services.ServiceCommand;
import org.robokind.impl.messaging.config.RKMessagingConfigUtils;
import static org.robokind.demo.robot.replication.lifecycle.ReplicationReceiverGroupLifecycleUtils.*;

/**
 *
 * @author Matthew Stevenson
 */
public class ReceiverTest {
    
    private final static String theGroupId = "receiverGroup";
    private final static String theConnectConfigId = "testConnectConfig";
    private final static String theDestConfigId = "receiverDestConfig";
    
    private final static String theRouteConsumerIdKey = "routeConsumerId";
    public static <T> void createRoute(
            String theRoutingKey, String theEncodeConfigId,
            ConsumerNode<Event<Header<String,Long>,T>> logger, 
            ManagedServiceFactory fact) {
        List<Configuration<String>> configs = Arrays.asList(
                getReplicationRouteLinkLifecycleConfig(
                        theGroupId, theRoutingKey +"LinkId", theRoutingKey, 
                        theRouteConsumerIdKey, theRoutingKey + "ConsumerId"),
                getReplicationDecoderLinkLifecycleConfig(
                        theGroupId, theRoutingKey + "DecoderId", theEncodeConfigId),
                RKManagedGroupConfigUtils.makeSimpleSelfBuildingLifecycle(
                        new String[]{ConsumerNode.class.getName()}, 
                        logger,
                        theRouteConsumerIdKey, theRoutingKey + "ConsumerId", null));
        Configuration<String> groupConf = 
                RKManagedGroupConfigUtils.buildManagedGroupConfig(
                        theGroupId, null, configs);
        ManagedServiceGroup group = 
                RKManagedGroupConfigUtils.buildGroup(fact, groupConf);
        group.start();
    }
    
    public static class TestEventLogger extends 
            DefaultConsumerNode<Event<Header<String,Long>,ServiceCommand>> {
        public TestEventLogger(){
            super(new Listener<Event<Header<String,Long>, ServiceCommand>>() {
                @Override
                public void handleEvent(Event<Header<String,Long>, ServiceCommand> input) {
                    Header<String,Long> h = input.getHeader();
                    ServiceCommand s = input.getData();
                    long now = System.currentTimeMillis();
                    System.out.println(
                            "Event Type: " + h.getEventType() + "\n"
                            + "Timestamp: " + h.getTimestamp() + " (elapsed: " 
                            + (now-h.getTimestamp()) + ", now:" + now + ")\n"
                            + "Source Reference: " + h.getSourceReference() + "\n"
                            + "Command Type: " + s.getCommand() + "\n");
                }
            });
            start();
        }
    }
    
    public static class TestErrorLogger extends 
            DefaultConsumerNode<Event<Header<String,Long>,ServiceError>> {
        public TestErrorLogger(){
            super(new Listener<Event<Header<String,Long>, ServiceError>>() {
                @Override
                public void handleEvent(Event<Header<String,Long>, ServiceError> input) {
                    Header<String,Long> h = input.getHeader();
                    ServiceError e = input.getData();
                    long now = System.currentTimeMillis();
                    System.out.println(
                            "Event Type: " + h.getEventType() + "\n"
                            + "Timestamp: " + h.getTimestamp() + " (elapsed: " 
                            + (now-h.getTimestamp()) + ", now:" + now + ")\n"
                            + "Source Reference: " + h.getSourceReference() + "\n"
                            + "Error Type: " + e.getErrorType() + "\n"
                            + "Error Message: " + e.getMessage() + "\n"
                            + "Error Details: " + e.getDetails() + "\n");
                }
            });
            start();
        }
    }
    
    public static void start(ManagedServiceFactory fact){
        RKMessagingConfigUtils.registerQueueConfig(theDestConfigId, "test", null, fact);
        List<Configuration<String>> lifecycles = 
                getLifecycleConfigs(theGroupId, theConnectConfigId, theDestConfigId);
        Configuration<String> groupConf = 
                RKManagedGroupConfigUtils.buildManagedGroupConfig(
                        theGroupId, null, lifecycles);
        ManagedServiceGroup group = RKManagedGroupConfigUtils.buildGroup(fact, groupConf);
        group.start();        
    }
    
    private static List<Configuration<String>> getLifecycleConfigs(
            String groupId, String connectionConfigId, String destConfigId){
        return new ListMergeAdapter().adapt(Arrays.asList(
                getJMSLifecycleConfigs(groupId, destConfigId, connectionConfigId),
                getReplicationReceiverLifecycleConfigs(groupId)
            ));
    }
}
